/*
 * プロフィール情報とQRコードを表示する画面
 * 
 */
package net.karappo.android.myqr;

import java.io.UnsupportedEncodingException;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.DialogInterface.OnDismissListener;
import android.os.Bundle;
import android.provider.Contacts.Intents;
import android.telephony.TelephonyManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SubMenu;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

public class MyQRActivity extends Activity
{
	private static final String TAG = "MyQR";
	
	private EditActivity editDialog;
	
	// PreferenceFile
	private static final String PREF_FILE_NAME = "ProfileInfoFile";
	SharedPreferences profileInfoPref;
	SharedPreferences.Editor editor;
	
	// -----------------------------------------
	// PreferenceFileへの保存キー
	
	public static final String PREF_NAME_LAST = "pref_name_last";
	public static final String PREF_NAME_FIRST = "pref_name_first";
	public static final String PREF_SOUND_LAST = "pref_sound_last";
	public static final String PREF_SOUND_FIRST = "pref_sound_first";
	public static final String PREF_TEL = "pref_tel";
	public static final String PREF_TEL_2 = "pref_tel_2";
	public static final String PREF_TEL_2_VISIBLE = "pref_tel_2_visible";
	public static final String PREF_EMAIL = "pref_email";
	public static final String PREF_EMAIL_2 = "pref_email_2";
	public static final String PREF_EMAIL_2_VISIBLE = "pref_email_2_visible";
	public static final String PREF_CARRIER = "pref_carrier";
	public static final String PREF_ADDRESS = "pref_address";
	public static final String PREF_ADDRESS_VISIBLE = "pref_address_visible";
	
	// -----------------------------------------
	// 保存する情報

	// プロフィール情報
	private String name_last;
	private String name_first;
	private String sound_last;
	private String sound_first;
	private String tel;
	private String tel_2;
	private String email;
	private String email_2;
	private String address;
	
	// キャリアー
	private int carrier;
	
	// 第２の表示非表示
	private Boolean tel_2_visible;
	private Boolean email_2_visible;
	private Boolean address_visible;
	
	// -----------------------------------------
	
	
	public static final int CA_DOCOMO 	= 0;
	public static final int CA_AU 		= 1;
	public static final int CA_SOFTBANK = 2;
	
	
	
	// UIへの参照
	LinearLayout layout_qr;
	TextView tv_name_first;
	TextView tv_name_last;
	TextView tv_name_sound_first;
	TextView tv_name_sound_last;
	TextView tv_tel_text;
	TextView tv_tel_text_2;
	TextView tv_mail_text;
	TextView tv_mail_text_2;
	TextView tv_carrier;
	TextView tv_address;
	
	public int MAX_TEXT_DATA_LENGTH = 120;

	// OptiojnMenuのSubMenu
	private SubMenu subMenu;

	// EditActivityが終了したら呼び出される用に
	private static final int LAUNCHED_ACTIVITY = 1;
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		
		profileInfoPref = getSharedPreferences(PREF_FILE_NAME , MODE_PRIVATE);
        editor = profileInfoPref.edit();
		
        // 画面の描画
        loadAndSetContentView();
        
        // 情報を取得
		loadData();
		
		// 電話番号がない場合はデバイスから取得
		if( (tel.length() == 0) || (tel == null) )
		{	
			TelephonyManager tm = (TelephonyManager) getSystemService(Context.TELEPHONY_SERVICE);
			tel = tm.getLine1Number();
			Log.d(TAG, "tel load from device"+tel);
		}
		
		
		// Profile情報の有無で場合分け
		
		if( profileIsEmpty() )
		{
			Log.d(TAG,"profile is empty > show edit dialog");
			
			startEdit();
		}
		else
		{
			Log.d(TAG,"profile is not empty > show QR and profiles");
			setProfileView();
			setQRView();
		}
		
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent intent) 
	{
		if(requestCode == LAUNCHED_ACTIVITY)
		{
			// EditActivityから戻ってきたら
			if(resultCode != RESULT_OK)
			{
				// エラー
				Log.e(TAG, "requestCode is not OK !");
			}
			else 
			{
				Log.d(TAG, "EditActivity has finished ...");
				
				name_last 		= intent.getStringExtra(PREF_NAME_LAST);
				name_first 		= intent.getStringExtra(PREF_NAME_FIRST);
				sound_last 		= intent.getStringExtra(PREF_SOUND_LAST);
				sound_first 	= intent.getStringExtra(PREF_SOUND_FIRST);
				tel 			= intent.getStringExtra(PREF_TEL);
				tel_2 			= intent.getStringExtra(PREF_TEL_2);
				tel_2_visible 	= intent.getBooleanExtra(PREF_TEL_2_VISIBLE, tel_2_visible);
				email 			= intent.getStringExtra(PREF_EMAIL);
				email_2 		= intent.getStringExtra(PREF_EMAIL_2);
				email_2_visible = intent.getBooleanExtra(PREF_EMAIL_2_VISIBLE, email_2_visible);
				carrier 		= intent.getIntExtra(PREF_CARRIER,carrier);
				address 		= intent.getStringExtra(PREF_ADDRESS);
				address_visible = intent.getBooleanExtra(PREF_ADDRESS_VISIBLE,address_visible);
				
				update();
			}
		}
		else
		{
			super.onActivityResult(requestCode, resultCode, intent);
		}
	}
	
	// EditActivityの起動
	private void startEdit()
	{
		Log.d(TAG, "ready to start EditActivity...");
		
		Intent intent = new Intent(getApplicationContext(),EditActivity.class);
		
		intent.putExtra(PREF_NAME_LAST, name_last);
		intent.putExtra(PREF_NAME_FIRST, name_first);
		intent.putExtra(PREF_SOUND_LAST, sound_last);
		intent.putExtra(PREF_SOUND_FIRST, sound_first);
		intent.putExtra(PREF_TEL, tel);
		intent.putExtra(PREF_TEL_2, tel_2);
		intent.putExtra(PREF_TEL_2_VISIBLE, tel_2_visible);
		intent.putExtra(PREF_EMAIL, email);
		intent.putExtra(PREF_EMAIL_2, email_2);
		intent.putExtra(PREF_EMAIL_2_VISIBLE, email_2_visible);
		intent.putExtra(PREF_CARRIER, carrier);
		intent.putExtra(PREF_ADDRESS, address);
		intent.putExtra(PREF_ADDRESS_VISIBLE, address_visible);
		
		Log.d(TAG, "start EditActivity...");
		
		startActivityForResult(intent,LAUNCHED_ACTIVITY);
	}
	
	private void update()
	{
		Log.d(TAG,"MyQRActivity update()");
		
		
		// QRの画像を置き換えることができないので、先に全体を読み込み直す必要がある
		loadAndSetContentView();
		
		// プロフィールの表示
		setProfileView();
		
		// QRの表示
		setQRView();
	}
	
	// 全体のレイアウトを読み込み
	private void loadAndSetContentView()
	{
		Log.d(TAG,"MyQRActivity loadAndSetContentView()");
		// レイアウトXML読み込み
		setContentView(R.layout.result);
		
		// UIへの参照を取得
		layout_qr = (LinearLayout) findViewById(R.id.qr_image);
		tv_name_first = (TextView) findViewById(R.id.name_first);
		tv_name_last = (TextView) findViewById(R.id.name_last);
		tv_name_sound_first = (TextView) findViewById(R.id.name_sound_first);
		tv_name_sound_last = (TextView) findViewById(R.id.name_sound_last);
		tv_tel_text = (TextView) findViewById(R.id.tel_text);
		tv_tel_text_2 = (TextView) findViewById(R.id.tel_text_2);
		// TODO PhoneNumberUtils.formatNumber(phone);を使って整形する
		tv_mail_text = (TextView) findViewById(R.id.mail_text);
		tv_mail_text_2 = (TextView) findViewById(R.id.mail_text_2);
		tv_carrier = (TextView) findViewById(R.id.carrier_txt);
		tv_address = (TextView) findViewById(R.id.address_text);
	}
	
	// Profile情報をViewに反映する
	private void setProfileView()
	{
		Log.d(TAG,"MyQRActivity setProfileView()");
		// UIを書き替え
		tv_name_first.setText(name_first);
		tv_name_last.setText(name_last);
		tv_name_sound_first.setText(sound_first);
		tv_name_sound_last.setText(sound_last);
		tv_tel_text.setText(tel);
		tv_tel_text_2.setText(tel_2);
		tv_mail_text.setText(email);
		tv_mail_text_2.setText(email_2);
		tv_address.setText(address);
		
		// 表示非表示切り替え
		
		// 名前
//		if(((name_first==null)||(name_first.length()==0))&&((name_last==null)||(name_last.length()==0)))
//			findViewById(R.id.TR_name).setVisibility(View.GONE);
//		else
//			findViewById(R.id.TR_name).setVisibility(View.VISIBLE);
		
		
		// ヨミガナ
//		if(((sound_first==null)||(sound_first.length()==0))&&((sound_last==null)||(sound_last.length()==0)))
//			findViewById(R.id.TR_sound).setVisibility(View.GONE);
//		else
//			findViewById(R.id.TR_sound).setVisibility(View.VISIBLE);
		
		// 電話
		if((tel==null)||(tel.length()==0))
			findViewById(R.id.TR_tel).setVisibility(View.GONE);
		else
			findViewById(R.id.TR_tel).setVisibility(View.VISIBLE);
		
		// 電話2
		if((tel_2==null)||(tel_2.length()==0)||!tel_2_visible)
			findViewById(R.id.TR_tel_2).setVisibility(View.GONE);
		else
			findViewById(R.id.TR_tel_2).setVisibility(View.VISIBLE);
		
		// メール
		if((email==null)||(email.length()==0))
		{
//			Log.d(TAG,">>>><<<<<<");
			findViewById(R.id.TR_email).setVisibility(View.GONE);
		}
		else
			findViewById(R.id.TR_email).setVisibility(View.VISIBLE);
		
		// メール2
		if((email_2==null)||(email_2.length()==0)||!email_2_visible)	
			findViewById(R.id.TR_email_2).setVisibility(View.GONE);
		else
			findViewById(R.id.TR_email_2).setVisibility(View.VISIBLE);
		
		// 住所
		if((address==null)||(address.length()==0)||!address_visible)	
			findViewById(R.id.TR_address).setVisibility(View.GONE);
		else
			findViewById(R.id.TR_address).setVisibility(View.VISIBLE);
	}
	
	// QR作成と描画、キャリア文字の変更
	private void setQRView()
	{	
		Log.d(TAG,"MyQRActivity setQRView()");
		// carrierで文字列を切り替え
		String text_data = "";
		switch (carrier) {
		case CA_DOCOMO:
			tv_carrier.setText("docomo");
	    	text_data = "MECARD:N:"+name_last+","+name_first+";SOUND:"+sound_last+","+sound_first+";";
	    	if( (tel.length()!=0) && (tel!=null) ) text_data += "TEL:"+tel+";";
	    	if( (tel_2.length()!=0) && (tel_2!=null) && tel_2_visible ) text_data += "TEL:"+tel_2+";";
	    	if( (email.length()!=0) && (email!=null) ) text_data += "EMAIL:"+email+";";
	    	if( (email_2.length()!=0) && (email_2!=null) && email_2_visible ) text_data += "EMAIL:"+email_2+";";
	    	if( (address.length()!=0) && (address!=null) && address_visible ) text_data += "MEMO:"+address+";";
	    	text_data += ";";
	    	break;
		case CA_AU:
			tv_carrier.setText("au");
			text_data = "MEMORY:\nNAME1:"+name_last+name_first+"\nNAME2:"+sound_last+sound_first;
			if( (tel.length()!=0) && (tel!=null) ) text_data += "\nTEL1:"+tel;
			if( (tel_2.length()!=0) && (tel_2!=null) && tel_2_visible ) text_data += "\nTEL2:"+tel_2;
			if( (email.length()!=0) && (email!=null) )text_data += "\nMAIL1:"+email;
			if( (email_2.length()!=0) && (email_2!=null) && email_2_visible ) text_data += "\nMAIL2:"+email_2;
			if( (address.length()!=0) && (address!=null) && address_visible ) text_data += "\nADD:"+address;
			break;
		case CA_SOFTBANK:
			tv_carrier.setText("softbank");
			text_data = "MEMORY:\nNAME1:"+name_last+name_first+"\nNAME2:"+sound_last+sound_first;
			if( (tel.length()!=0) && (tel!=null) ) text_data += "\nTEL1:"+tel;
			if( (tel_2.length()!=0) && (tel_2!=null) && tel_2_visible ) text_data += "\nTEL2:"+tel_2;
			if( (email.length()!=0) && (email!=null) )text_data += "\nMAIL1:"+email;
			if( (email_2.length()!=0) && (email_2!=null) && email_2_visible ) text_data += "\nMAIL2:"+email_2;
			if( (address.length()!=0) && (address!=null) && address_visible ) text_data += "\nADD:"+address;
			break;
		default:
			break;
		}
		
		
		// 文字数を調べる
		byte[] d = null;
		try {
			d = text_data.getBytes("Shift-JIS");
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		Log.d(TAG,"text_data : "+text_data);
		Log.d(TAG,"text_data.length : "+d.length);
		
		if(d.length < MAX_TEXT_DATA_LENGTH)
		{
			// QR作成
			QRView qrView = new QRView(this, text_data );
			
			// QRを表示
			layout_qr.addView(qrView);
		}
		else
		{
			Log.e(TAG, "QR code generate FAILED![ text_data.length="+text_data.length()+" / MAX_TEXT_DATA_LENGTH="+MAX_TEXT_DATA_LENGTH);
			
			// アラーとダイアログの作成
			new AlertDialog.Builder(MyQRActivity.this)
            .setIcon(R.drawable.alert_dialog_icon)
            .setTitle("QRコード作成に失敗")
            .setMessage(
		            		"現在データ量（"+d.length+"）が限界（"+MAX_TEXT_DATA_LENGTH+"）を超えたため、QRコードの作成に失敗しました。\n" +
		            		"申し訳ありませんがやり直してください。")
            .setPositiveButton("無視する", new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {
                	// 何もしない
                }
            })
            .setNegativeButton("やり直す", new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {
                	// もう一度編集画面に戻る
                	startEdit();
                }
            })
            .create()
            .show();
;
			
//	    	Toast.makeText(MyQRActivity.this, "最大文字数"+MAX_TEXT_DATA_LENGTH+"を超えていたため、QRコードの作成に失敗しました。\n文字数："+text_data.length()+"\n不要な番号などは非表示にしておくと文字数が節約できます。", Toast.LENGTH_LONG).show();
		}
		
	}
	
	
	//プロフィール情報が登録されているかを半別
	private boolean profileIsEmpty()
	{
		String [] prof = { name_last, name_first, sound_last, sound_first, email };
		for (int i = 0; i < prof.length; i++) if(prof[i].length() != 0) return false;
		return true;
	}



	@Override 
	public boolean onCreateOptionsMenu(Menu menu)
	{
		super.onCreateOptionsMenu(menu);
		
		subMenu = menu.addSubMenu(0,0,0,R.string.title_carrier).setIcon(R.drawable.carrier_icon);
		subMenu.add(1,10,0,R.string.carrier_name_docomo);
		subMenu.add(1,11,0,R.string.carrier_name_au);
		subMenu.add(1,12,0,R.string.carrier_name_softbank);
		subMenu.setGroupCheckable(1, true, true);
		menu.add(0,1,0, R.string.title_edit).setIcon(R.drawable.edit_icon);
		menu.add(0,2,0, R.string.title_info).setIcon(R.drawable.info_icon);
		return true;
	}
	
	public boolean onPrepareOptionsMenu(Menu menu)
	{
		super.onCreateOptionsMenu(menu);
		int id = carrier+10;
//		Log.d(TAG,">>>>>>>>>> carrier:"+carrier+",id:"+id);
		subMenu.findItem(id).setChecked(true);	// 現在のキャリアにチェック
		return true;
	}
	
	@Override 
	public boolean onOptionsItemSelected(MenuItem item)
	{
//		Log.d(TAG, ">>>>>>>>>>>>>>>>>>>>>>>> "+item.getItemId());
		switch (item.getItemId()) {
		case 1:
			startEdit();
			return true;
		case 10:
			carrier = CA_DOCOMO;
			update();
			return true;
		case 11:
			carrier = CA_AU;
			update();
			return true;
		case 12:
			carrier = CA_SOFTBANK;
			update();
			return true;
		case 2:
			// アラーとダイアログの作成
			new AlertDialog.Builder(MyQRActivity.this)
            .setTitle(R.string.info_title)
            .setMessage(R.string.info_text)
            .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {
                	// 何かする場合はここ
                }
            })
            .create()
            .show();
		}
		return false;
	}
	
	@Override
	protected void onDestroy() 
	{
		saveData();
		super.onDestroy();
	}
	
	
	// ----------------------------------------------------------------------------------
	// referenceFileからの読み込み関係
	// ----------------------------------------------------------------------------------
	
	private int getIntValue(String pref_key)
    {
    	// 値がない場合はCA_DOCOMOを返す
		int val = profileInfoPref.getInt(pref_key, CA_DOCOMO);
    	Log.d(TAG,"load from Preference File "+pref_key+" : "+val);
    	return val;
    }
	private void setIntValue(String pref_key, int val)
	{
		Log.d(TAG,"save to Preference File "+pref_key+" : "+val);
		editor.putInt(pref_key , val);
	}
    private String getValue(String pref_key)
    {
    	// 値がない場合は""を返す
    	String val = profileInfoPref.getString(pref_key, "");
    	Log.d(TAG,"load from Preference File "+pref_key+" : "+val);
    	return val;
    }
    private void setValue(String pref_key, String val)
	{
		Log.d(TAG,"save to Preference File "+pref_key+" : "+val);
		editor.putString(pref_key , val);
	}
    private Boolean getBooleanValue(String pref_key)
    {
    	// 値がない場合はfalseを返す
		Boolean val = profileInfoPref.getBoolean(pref_key, false);
    	Log.d(TAG,"load from Preference File "+pref_key+" : "+val);
    	return val;
    }
	private void setBooleanValue(String pref_key, Boolean val)
	{
		Log.d(TAG,"save to Preference File "+pref_key+" : "+val);
		editor.putBoolean(pref_key , val);
	}
    
	// プロフィール情報の保存
	private void saveData()
	{
		Log.d(TAG, "save prozfiles into preference file");
		
		setValue(PREF_NAME_LAST , name_last);
		setValue(PREF_NAME_FIRST , name_first);
		setValue(PREF_SOUND_LAST , sound_last);
		setValue(PREF_SOUND_FIRST , sound_first);
		setValue(PREF_TEL , tel);
		setValue(PREF_TEL_2 , tel_2);
		setBooleanValue(PREF_TEL_2_VISIBLE , tel_2_visible);
		setValue(PREF_EMAIL , email);
		setValue(PREF_EMAIL_2 , email_2);
		setBooleanValue(PREF_EMAIL_2_VISIBLE , email_2_visible);
		setValue(PREF_ADDRESS , address);
		setBooleanValue(PREF_ADDRESS_VISIBLE , address_visible);
		
		setIntValue(PREF_CARRIER , carrier);
		
		Toast.makeText(MyQRActivity.this, "MyQR データを保存しました", Toast.LENGTH_SHORT).show();
		
		editor.commit();
	}
	
	// プロフィール情報の読み込み
	private void loadData()
	{
		name_last 		= getValue(PREF_NAME_LAST);
		name_first 		= getValue(PREF_NAME_FIRST);
		sound_last 		= getValue(PREF_SOUND_LAST);
		sound_first 	= getValue(PREF_SOUND_FIRST);
		email 			= getValue(PREF_EMAIL);
		email_2 		= getValue(PREF_EMAIL_2);
		email_2_visible = getBooleanValue(PREF_EMAIL_2_VISIBLE);
		tel 			= getValue(PREF_TEL);
		tel_2 			= getValue(PREF_TEL_2);
		tel_2_visible 	= getBooleanValue(PREF_TEL_2_VISIBLE);
		address 		= getValue(PREF_ADDRESS);
		address_visible = getBooleanValue(PREF_ADDRESS_VISIBLE);

		carrier = getIntValue(PREF_CARRIER);
	}
	
}
