/*
 * QRコードを描画するクラス
 * 
 */

package net.karappo.android.myqr;

import java.io.UnsupportedEncodingException;

import com.swetake.util.Qrcode;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.util.Log;
import android.view.View;

public class QRView extends View 
{
	private static final String TAG = "MyQR";
	
	private boolean[][] matrix;	// QRマトリクス
	private static int UNIT_SCALE = 4;			// １ユニットの大きさ（pix）QRの大きさはこれ×45
	private static int MARGINE_WIDTH = 23;		// 外側のマージン
	private static int IMAGE_WIDTH = 226;
	private static int IMAGE_HEIGHT = 226;
	
	public QRView(Context context, String src_text) 
	{
		super(context);
		matrix = getMatrix(src_text);
	}
	
	@Override
	protected void onDraw(Canvas canvas) 
	{
//		Log.d(TAG, "onDraw()");
		
		// 地
		Paint ground = new Paint();
		ground.setColor(getResources().getColor(R.color.ground_color));
		canvas.drawRect(0,0,IMAGE_WIDTH, IMAGE_HEIGHT, ground);
		
		// 図
		Paint figure = new Paint();
		figure.setColor(getResources().getColor(R.color.figure_color));
		
//		Log.d(TAG, "matrix="+matrix);
//		Log.d(TAG, "matrix.length="+matrix.length);
		
    	for(int i=0; i<matrix.length; i++)
        {
        	for(int j=0; j<matrix.length; j++)
        	{
        		if(matrix[j][i])
        		{
        			canvas.drawRect(MARGINE_WIDTH+j*UNIT_SCALE,MARGINE_WIDTH+i*UNIT_SCALE,MARGINE_WIDTH+(j+1)*UNIT_SCALE,MARGINE_WIDTH+(i+1)*UNIT_SCALE, figure);
        		}
        	}
        }
		super.onDraw(canvas);
	}
	
	// ----------------------------------------------------------------------------------
	// QRコードのマトリクス作成
	// ----------------------------------------------------------------------------------
	
	private  boolean[][] getMatrix( String _qrdata )
	{
		Qrcode x = new Qrcode();
        x.setQrcodeErrorCorrect('M');
        x.setQrcodeEncodeMode('B');
        x.setQrcodeVersion(7);
       
		try {
			byte[] d = _qrdata.getBytes("Shift-JIS");
			if(d.length>0 && d.length<120 )
	        {
				boolean [][] matrix = x.calQrcode(d);
				Log.d(TAG, "[OK] matrix has returned");
				return matrix;
	        }
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
			Log.e(TAG, "[Error] Can't change text data to matrix !");
		}
//		Log.e(TAG, "------------------------------------------------ return null");
		return null;
	}
}
